<?php

/**
 * @author PayAnyWay
 * @copyright 2018
 * @package AppEngine Payment
 */

/*
    Plugin Name: FrE PayAnyWay
    Plugin URI: https://payanyway.ru/info/w/ru/public/w/partnership/developers/cms.html
    Description: Integrates the PayAnyWay payment gateway to your FreelanceEngine site
    Version: 1.0.0
    Author: PayAnyWay
    Author URI: https://www.payanyway.ru/
    License: GPLv2
    Text Domain: enginetheme
*/

$payanyway_order = null;

function ae_payanyway_require_plugin_file(){
    if(!class_exists('AE_Base')) return;
}

add_action('after_setup_theme', 'ae_payanyway_require_plugin_file');
//setup admin option
add_filter('ae_admin_menu_pages', 'ae_payanyway_add_settings');

function ae_payanyway_add_settings($pages)
{
    $sections = array();
    $options = AE_Options::get_instance();

    /**
     * ae fields settings
     */
    $sections = array(
        'args' => array(
            'title' => __("PayAnyWay API", ET_DOMAIN) ,
            'id' => 'payanyway_field',
            'icon' => 'F',
            'class' => ''
        ) ,

        'groups' => array(
            array(
                'args' => array(
                    'title' => __("Настройка платёжного шлюза PayAnyWay", ET_DOMAIN) ,
                    'id' => 'payanyway',
                    'class' => '',
                    'desc' => __('Заполните поля в соответствии с настройками вашего расширенного счёта', ET_DOMAIN) ,
                    'name' => 'payanyway'
                ) ,
                'fields' => array(
                    array(
                        'id' => 'payanyway_mnt_id',
                        'type' => 'text',
                        'label' => __("Номер расширенного счёта", ET_DOMAIN) ,
                        'name' => 'payanyway_mnt_id',
                        'class' => ''
                    ) ,
                    array(
                        'id' => 'payanyway_mnt_secret',
                        'type' => 'text',
                        'label' => __("Код проверки целостности данных", ET_DOMAIN) ,
                        'name' => 'payanyway_mnt_secret',
                        'class' => ''
                    ) ,
                    array(
                        'id' => 'payanyway_assistant_url',
                        'type' => 'select',
                        'label' => __("Сервер", ET_DOMAIN) ,
                        'name' => 'payanyway_assistant_url',
                        'data' => array
                        (
                            'https://www.payanyway.ru/assistant.htm' => 'www.payanyway.ru',
                            'https://demo.moneta.ru/assistant.htm' => 'demo.moneta.ru',
//                            'http://localhost:8080/assistant.htm' => 'AssistEmu'
                        ),
                        'class' => ''
                    )
                )
            )
        )
    );

    $temp = new AE_section($sections['args'], $sections['groups'], $options);

    $payanyway_setting = new AE_container(array(
        'class' => 'field-settings',
        'id' => 'settings',
    ) , $temp, $options);

    $pages[] = array(
        'args' => array(
            'parent_slug' => 'et-overview',
            'page_title' => __('PayAnyWay', ET_DOMAIN) ,
            'menu_title' => __('PAYANYWAY', ET_DOMAIN) ,
            'cap' => 'administrator',
            'slug' => 'ae-payanyway',
            'icon' => '$',
            'icon_class' => 'fa fa-inr',
            'desc' => __("Integrate the PayAnyWay payment gateway to your site", ET_DOMAIN)
        ) ,
        'container' => $payanyway_setting
    );

    return $pages;
}

add_filter('ae_support_gateway', 'ae_payanyway_add_support');
function ae_payanyway_add_support($gateways)
{
    $gateways['payanyway'] = 'PayAnyWay';
    return $gateways;
}

//add button front end
add_action('after_payment_list', 'ae_payanyway_render_button');
add_action('after_payment_list_upgrade_account', 'ae_payanyway_render_button');

function ae_payanyway_render_button()
{
    $payanyway_key = ae_get_option('payanyway');
    if ( empty($payanyway_key['payanyway_mnt_id']) || empty($payanyway_key['payanyway_mnt_secret']) || empty($payanyway_key['payanyway_assistant_url']) ) return false;
?>
    <li class="panel">
        <span class="title-plan payanyway-payment" data-type="payanyway">
            <?php _e("PayAnyWay", ET_DOMAIN); ?>
            <span><?php _e("Send your payment to our PayAnyWay account", ET_DOMAIN); ?></span>
        </span>
        <?php _e("Select", ET_DOMAIN); ?>
        <a data-toggle="collapse" data-type="payanyway" data-parent="#fre-payment-accordion" href="#fre-payment-payanyway" class="btn collapsed other-payment"><?php _e("Select", ET_DOMAIN); ?></a>
        <?php include_once dirname(__FILE__) . '/form-template.php'; ?>
    </li>
<?php
}

add_action('wp_print_scripts', 'ae_payanyway_script');
function ae_payanyway_script()
{
    if (is_page_template('page-post-place.php') || is_page_template('page-submit-project.php')) {
        wp_enqueue_script('ae_payanyway', plugin_dir_url(__FILE__) . 'assets/payanyway.js', array(
            'underscore',
            'backbone',
            'appengine'
        ) , '1.0', true);
        $test_mode = ET_Payment::get_payment_test_mode();
        $mnt_test_mode = $test_mode ? '1' : '0';
        wp_localize_script('ae_payanyway', 'ae_payanyway', array(
            'mnt_test_mode' => $mnt_test_mode,
            'mnt_currency_code' => ae_get_option('currency'),
        ));
    }
}

add_action('ae_payment_script', 'ae_payanyway_add_script');
function ae_payanyway_add_script()
{
    wp_enqueue_script('ae_payanyway', plugin_dir_url(__FILE__) . 'assets/payanyway.js', array(
        'underscore',
        'backbone',
        'appengine'
    ) , '1.0', true);
    $test_mode = ET_Payment::get_payment_test_mode();
    $mnt_test_mode = $test_mode ? '1' : '0';
    wp_localize_script('ae_payanyway', 'ae_payanyway', array(
        'mnt_test_mode' => $mnt_test_mode,
        'mnt_currency_code' => ae_get_option('currency'),
    ));
}

add_filter('ae_setup_payment', 'ae_payanyway_setup_payment', 10, 3);

/**
 * @param $response
 * @param $paymentType
 * @param $order AE_Order
 *
 * @return array
 */
function ae_payanyway_setup_payment($response, $paymentType, $order)
{
    if ($paymentType == 'PAYANYWAY')
    {
        $order_pay = $order->generate_data_to_pay();
        $order_id = $order_pay['ID'];
        $payanyway_info = ae_get_option('payanyway');
        $productinfo = array_values($order_pay['products']);
        $test_mode = ET_Payment::get_payment_test_mode();
        $hash_data['mnt_id'] = $payanyway_info['payanyway_mnt_id'];
        $hash_data['mnt_amount'] = number_format($productinfo[0]['AMT'], 2, '.', '');
        $hash_data['mnt_transaction_id'] = $order_id;
        $hash_data['mnt_currency_code'] = $order->get_order_data()['currency'];
        $hash_data['mnt_description'] = $productinfo[0]['ID']." ".$productinfo[0]['NAME'];
        $hash_data['mnt_test_mode'] = $test_mode ? '1' : '0';
        $hash_data['mnt_signature'] = ae_paw_calculate_hash_before_transaction($hash_data);
        $response = array(
            'success' => true,
            'data' => array(
                'ACK' => true,
                'data_arr' => $hash_data
            ) ,
            'paymentType' => 'PAYANYWAY'
        );
    }
    return $response;
}

register_activation_hook(__FILE__, 'payanyway_activate');
function payanyway_activate() {
    create_custom_page('payanyway_payurl');
}

function create_custom_page($page_name) {
    $pageExists = false;
    $pages = get_pages();
    foreach ($pages as $page) {
        if ($page->post_name == $page_name) {
            $pageExists = true;
            break;
        }
    }
    if (!$pageExists) {
        wp_insert_post ([
            'post_type' =>'page',
            'post_name' => $page_name,
            'post_status' => 'publish',
        ]);
    }
}
// End Plugin Activation


add_action( 'init', 'payanyway_payurl_init_internal' );
add_filter( 'query_vars', 'payanyway_payurl_query_vars' );
add_action( 'parse_request', 'payanyway_payurl_parse_request' );
function payanyway_payurl_init_internal() { add_rewrite_rule( '', 'index.php?paw_payurl=1', 'top' ); }
function payanyway_payurl_query_vars( $query_vars )
{
    $query_vars[] = 'paw_payurl';
    $query_vars[] = 'MNT_ID';
    $query_vars[] = 'MNT_TRANSACTION_ID';
    $query_vars[] = 'MNT_OPERATION_ID';
    $query_vars[] = 'MNT_AMOUNT';
    $query_vars[] = 'MNT_CURRENCY_CODE';
    $query_vars[] = 'MNT_TEST_MODE';
    $query_vars[] = 'MNT_SIGNATURE';
    return $query_vars;
}
function payanyway_payurl_parse_request( &$wp )
{
    if (array_key_exists('paw_payurl', $wp->query_vars ))
    {
        $fields = array(
            'mnt_id' => @$wp->query_vars['MNT_ID'],
            'mnt_transaction_id' => @$wp->query_vars['MNT_TRANSACTION_ID'],
            'mnt_operation_id' => @$wp->query_vars['MNT_OPERATION_ID'],
            'mnt_amount' => @$wp->query_vars['MNT_AMOUNT'],
            'mnt_currency_code' => @$wp->query_vars['MNT_CURRENCY_CODE'],
            'mnt_test_mode' => @$wp->query_vars['MNT_TEST_MODE']
        );
        $order = new AE_Order(@$wp->query_vars['MNT_TRANSACTION_ID']);
        if (is_object($order))
        {
            if (@$wp->query_vars['MNT_SIGNATURE'] === ae_verify_payanyway_hash($fields))
            {
                $order->set_status('publish');
                $order->update_order();
                echo 'SUCCESS';
                exit();
            }
        }

        echo 'FAIL';
        exit();
    }
    return;
}

//add_action('wp_footer', 'ae_payanyway_form_template');
//function ae_payanyway_form_template()
//{
//    include_once dirname(__FILE__) . '/form-template.php';
//}

function ae_verify_payanyway_hash($data_value)
{
    $payanyway_info = ae_get_option('payanyway');
    $hashSequence = "mnt_id|mnt_transaction_id|mnt_operation_id|mnt_amount|mnt_currency_code|mnt_test_mode";
    $hashVarsSeq = explode('|', $hashSequence);
    $hash_string = '';
    foreach ($hashVarsSeq as $hash_var)
        $hash_string .= isset($data_value[$hash_var]) ? $data_value[$hash_var] : '';
    $hash_string .= $payanyway_info['payanyway_mnt_secret'];
    return md5($hash_string);
}

function ae_paw_calculate_hash_before_transaction($hash_data)
{
    $payanyway_info = ae_get_option('payanyway');
    $hashSequence = "mnt_id|mnt_transaction_id|mnt_amount|mnt_currency_code|mnt_test_mode";
    $hashVarsSeq = explode('|', $hashSequence);
    $hash_string = '';
    foreach ($hashVarsSeq as $hash_var)
        $hash_string .= isset($hash_data[$hash_var]) ? $hash_data[$hash_var] : '';
    $hash_string .= $payanyway_info['payanyway_mnt_secret'];
    return md5($hash_string);
}
